package fib;

import org.junit.jupiter.api.AfterAll;
import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.BeforeAll;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;

// from https://www.petrikainulainen.net/programming/testing/junit-5-tutorial-writing-nested-tests/
@DisplayName("JUnit 5 Nested Example")
public class NestedTest {
	String toBeTested;
	
	@BeforeAll
	static void beforeAll() {
		System.out.println("Before all test methods");
	}

	@BeforeEach
	void beforeEach() {
		System.out.println("Before each test method");
		toBeTested = new String("xyz");
	}

	@AfterEach
	void afterEach() {
		System.out.println("After each test method");
	}

	@Test
	void t() {
		// test: use toBeTested
	}
	
	@AfterAll
	static void afterAll() {
		System.out.println("After all test methods");
	}

	@Nested
	@DisplayName("Tests for the method A")
	class A {

		@BeforeEach
		void beforeEach() {
			System.out.println("Before each test method of the A class");
		}

		@AfterEach
		void afterEach() {
			System.out.println("After each test method of the A class");
		}

		@Test
		@DisplayName("Example test for method A")
		void sampleTestForMethodA() {
			System.out.println("Example test for method A");
		}

		@Nested
		@DisplayName("When X is true")
		class WhenX {

			@BeforeEach
			void beforeEach() {
				System.out.println("Before each test method of the WhenX class");
			}

			@AfterEach
			void afterEach() {
				System.out.println("After each test method of the WhenX class");
			}

			@Test
			@DisplayName("Example test for method A when X is true")
			void sampleTestForMethodAWhenX() {
				System.out.println("Example test for method A when X is true");
			}
		}
	}
}